Crashkernel=X tries to reserve memory for the crash dump kernel under
4G. If crashkernel=X,low is specified simultaneously, reserve spcified
size low memory for crash kdump kernel devices firstly and then reserve
memory above 4G.

Suggested by James, just introduced crashkernel=X,low to arm64. As memtioned
above, if crashkernel=X,low is specified simultaneously, reserve spcified
size low memory for crash kdump kernel devices firstly and then reserve
memory above 4G, which is much simpler.

Signed-off-by: Chen Zhou <chenzho...@huawei.com>
Tested-by: John Donnelly <john.p.donne...@oracle.com>
Tested-by: Prabhakar Kushwaha <pkushw...@marvell.com>
---
 arch/arm64/kernel/setup.c |  8 +++++++-
 arch/arm64/mm/init.c      | 31 +++++++++++++++++++++++++++++--
 2 files changed, 36 insertions(+), 3 deletions(-)

diff --git a/arch/arm64/mm/init.c b/arch/arm64/mm/init.c
index 1e93cfc7c47a..ce7ced85f5fb 100644
--- a/arch/arm64/mm/init.c
+++ b/arch/arm64/mm/init.c
@@ -81,6 +81,7 @@ static void __init reserve_crashkernel(void)
 {
        unsigned long long crash_base, crash_size;
        int ret;
+       phys_addr_t crash_max = arm64_dma32_phys_limit;
 
        ret = parse_crashkernel(boot_command_line, memblock_phys_mem_size(),
                                &crash_size, &crash_base);
@@ -88,12 +89,38 @@ static void __init reserve_crashkernel(void)
        if (ret || !crash_size)
                return;
 
+       ret = reserve_crashkernel_low();
+       if (!ret && crashk_low_res.end) {
+               /*
+                * If crashkernel=X,low specified, there may be two regions,
+                * we need to make some changes as follows:
+                *
+                * 1. rename the low region as "Crash kernel (low)"
+                * In order to distinct from the high region and make no effect
+                * to the use of existing kexec-tools, rename the low region as
+                * "Crash kernel (low)".
+                *
+                * 2. change the upper bound for crash memory
+                * Set MEMBLOCK_ALLOC_ACCESSIBLE upper bound for crash memory.
+                *
+                * 3. mark the low region as "nomap"
+                * The low region is intended to be used for crash dump kernel
+                * devices, just mark the low region as "nomap" simply.
+                */
+               const char *rename = "Crash kernel (low)";
+
+               crashk_low_res.name = rename;
+               crash_max = MEMBLOCK_ALLOC_ACCESSIBLE;
+               memblock_mark_nomap(crashk_low_res.start,
+                                   resource_size(&crashk_low_res));
+       }
+
        crash_size = PAGE_ALIGN(crash_size);
 
        if (crash_base == 0) {
                /* Current arm64 boot protocol requires 2MB alignment */
-               crash_base = memblock_find_in_range(0, arm64_dma32_phys_limit,
-                               crash_size, SZ_2M);
+               crash_base = memblock_find_in_range(0, crash_max, crash_size,
+                               SZ_2M);
                if (crash_base == 0) {
                        pr_warn("cannot allocate crashkernel (size:0x%llx)\n",
                                crash_size);
diff --git a/arch/arm64/kernel/setup.c b/arch/arm64/kernel/setup.c
index 93b3844cf442..4dc51a2ac012 100644
--- a/arch/arm64/kernel/setup.c
+++ b/arch/arm64/kernel/setup.c
@@ -238,7 +238,13 @@ static void __init request_standard_resources(void)
                    kernel_data.end <= res->end)
                        request_resource(res, &kernel_data);
 #ifdef CONFIG_KEXEC_CORE
-               /* Userspace will find "Crash kernel" region in /proc/iomem. */
+               /*
+                * Userspace will find "Crash kernel" region in /proc/iomem.
+                * Note: the low region is renamed as Crash kernel (low).
+                */
+               if (crashk_low_res.end && crashk_low_res.start >= res->start &&
+                               crashk_low_res.end <= res->end)
+                       request_resource(res, &crashk_low_res);
                if (crashk_res.end && crashk_res.start >= res->start &&
                    crashk_res.end <= res->end)
                        request_resource(res, &crashk_res);
-- 
2.20.1


_______________________________________________
kexec mailing list
kexec@lists.infradead.org
http://lists.infradead.org/mailman/listinfo/kexec

Reply via email to